package kr.co.vividnext.sodalive.content.playlist

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.order.OrderRepository
import kr.co.vividnext.sodalive.live.roulette.RedisIdGenerator
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service

@Service
class AudioContentPlaylistService(
    private val idGenerator: RedisIdGenerator,
    private val orderRepository: OrderRepository,
    private val audioContentRepository: AudioContentRepository,
    private val redisRepository: AudioContentPlaylistRedisRepository
) {
    fun createPlaylist(request: CreatePlaylistRequest, member: Member) {
        if (request.contentIdList.size >= 30) {
            throw SodaException("플레이 리스트에는 최대 30개의 콘텐츠를 저장할 수 있습니다.")
        }

        val playlistCount = redisRepository.findByMemberId(member.id!!).size
        if (playlistCount >= 10) {
            throw SodaException("플레이 리스트는 최대 10개까지 생성할 수 있습니다.")
        }

        // 콘텐츠 유효성 검사 (소장으로 구매한 콘텐츠 인가?)
        checkOrderedContent(
            contentIdList = request.contentIdList,
            memberId = member.id!!
        )

        val playlist = AudioContentPlaylist(
            id = idGenerator.generateId(SEQUENCE_NAME),
            memberId = member.id!!,
            title = request.title,
            desc = request.desc,
            contentIdList = request.contentIdList
        )

        redisRepository.save(playlist)
    }

    private fun checkOrderedContent(contentIdList: List<Long>, memberId: Long) {
        val orderedContentIdList = orderRepository.findOrderedContent(contentIdList, memberId).toSet()
        val orderedContentMap = contentIdList.associateWith { it in orderedContentIdList }
        val notOrderedContentList = orderedContentMap.filterValues { !it }.keys

        if (notOrderedContentList.isNotEmpty()) {
            throw SodaException("소장하지 않은 콘텐츠는 재생목록에 추가할 수 없습니다.")
        }
    }

    fun getPlaylists(member: Member): GetPlaylistsResponse {
        val playlists = redisRepository.findByMemberId(memberId = member.id!!)

        return GetPlaylistsResponse(
            totalCount = playlists.size,
            items = playlists.map {
                val contentCount = it.contentIdList.size
                val coverImageUrl = if (contentCount > 0) {
                    audioContentRepository.getCoverImageById(id = it.contentIdList[0])
                        ?: ""
                } else {
                    ""
                }
                GetPlaylistsItem(
                    id = it.id,
                    title = it.title,
                    desc = it.desc ?: "",
                    contentCount = contentCount,
                    coverImageUrl = coverImageUrl
                )
            }
        )
    }

    fun deletePlaylist(playlistId: Long, member: Member) {
        val playlist = redisRepository.findByIdOrNull(id = playlistId)
            ?: throw SodaException("잘못된 요청입니다.")

        if (playlist.memberId != member.id) {
            throw SodaException("잘못된 요청입니다.")
        }

        redisRepository.delete(playlist)
    }

    companion object {
        const val SEQUENCE_NAME = "AudioContentPlaylist:sequence"
    }
}
